@@ -384,13 +384,26 @@ def exit_error(msg, exception=None):
 
     print(f"ERROR [in: {command}]: {msg[0]}", file=sys.stderr)
     print(file=sys.stderr)
+    print("----------- traceback -----------", file=sys.stderr)
+    print(traceback.print_stack(), file=sys.stderr)
+    print("---------------------------------", file=sys.stderr)
+    print(file=sys.stderr)
 
     for line in msg[1:]:
         print(f"  {line}", file=sys.stderr)
+
+    print(file=sys.stderr)
+
     print("exiting...", file=sys.stderr)
     sys.exit(EXIT_ERROR)
 
 
+def warn(msg):
+    """print a warning to stderr prefixed with WARNING:"""
+    msg = dedent(msg)
+    print(f"WARNING: {msg}", file=sys.stderr)
+
+
 def process_stream_and_add_frames(
     frames: List[Saveframe], input_args: Namespace
 ) -> Entry:
