@@ -6,6 +6,7 @@ from nef_pipelines.lib.test_lib import (
     assert_lines_match,
     isolate_frame,
     path_in_test_data,
+    read_test_data,
     run_and_report,
 )
 from nef_pipelines.lib.util import get_version
@@ -82,7 +83,7 @@ def test_3aa10():
     assert_lines_match(EXPECTED_3AA10, mol_sys_result)
 
 
-HEADER = open(path_in_test_data(__file__, "test_header_entry.txt")).read()
+HEADER = read_test_data("test_header_entry.txt", __file__)
 
 EXPECTED_HEADER = f"""\
 save_nef_nmr_meta_data
@@ -95,7 +96,6 @@ save_nef_nmr_meta_data
    _nef_nmr_meta_data.program_version  {get_version()}
    _nef_nmr_meta_data.creation_date    2012-01-14T12:00:01.123456
    _nef_nmr_meta_data.uuid             NEFPipelines-2012-01-14T12:00:01.123456-1043321819
-   _nef_nmr_meta_data.creation_time    2012-01-14T12:00:01.123456
 
    loop_
       _nef_run_history.run_number
@@ -117,7 +117,7 @@ def test_pipe_header(fixed_seed):
 
     path = path_in_test_data(__file__, "3aa10.seq")
     path_header = path_in_test_data(__file__, "test_header_entry.txt")
-    result = run_and_report(app, ["--pipe", path_header, path])
+    result = run_and_report(app, ["--in", path_header, path])
 
     mol_sys_result = isolate_frame(result.stdout, "%s" % NEF_MOLECULAR_SYSTEM)
     meta_data_result = isolate_frame(result.stdout, "%s" % NEF_METADATA)
