@@ -100,7 +100,10 @@ ABBREVIATED_HEADINGS = {
     "ccpn_figure_of_merit": "merit",
     "ccpn_annotation": "ann",
     "ccpn_peak_list_serial": "ccpn-serial",
-    "atom-name": "atom",
+    "atom_name": "atom",
+    "isotope_number": "iso",
+    "value_uncertainty": "err",
+    "element": "elem",
 }
 SELECT_HELP = """
     a list of columns to select for output, to specify multiple columns specify the option multiple times or use a comma
@@ -175,6 +178,8 @@ def tabulate(
 
     args.select_columns = _build_column_selections(args.select_columns, args.exact)
 
+    args.frame_loop_selectors = parse_comma_separated_options(args.frame_loop_selectors)
+
     entry = read_or_create_entry_exit_error_on_bad_file(args.input_file)
 
     tabulate_frames(entry, args)
@@ -375,8 +380,12 @@ def _output_loop(loop_data, frame_id, frame_category, entry_id, args, seen_files
             used_headers = _abbreviate_headers(used_headers, ABBREVIATED_HEADINGS)
 
         if not args.no_title:
-            print(frame_category)
-            print("-" * len(frame_category))
+            if len(frame_id.strip()) == 0:
+                title = f"{frame_category}"
+            else:
+                title = f"{frame_id} [{frame_category}]"
+            print(title)
+            print("-" * len(title))
             print()
 
         if args.out_format in ["csv", ""]:
@@ -393,7 +402,9 @@ def _output_loop(loop_data, frame_id, frame_category, entry_id, args, seen_files
                 file=out_file,
             )
 
-        if out_type != OutputFormat.STDOUT:
+        if out_type == OutputFormat.STDOUT:
+            print()
+        else:
             out_file.close()
 
 
