@@ -1,3 +1,5 @@
+import sys
+
 from nef_pipelines import nef_app
 
 if nef_app:
@@ -5,4 +7,6 @@ if nef_app:
     @nef_app.app.command()
     def sink():
         """- read the current stream and don't write anything"""
-        ...
+        if not sys.stdin.isatty():
+            for line in sys.stdin:
+                pass
