@@ -13,5 +13,5 @@ if nef_app.app:
     app.add_typer(import_app, name="import", help="- import [rdcs]")
 
     # import of specific importers must be after app creation to avoid circular imports
-    import nef_pipelines.transcoders.csv.importers.peaks  # noqa: F401
-    import nef_pipelines.transcoders.csv.importers.rdcs  # noqa: F401
+    import nef_pipelines.transcoders.csv.importers._peaks_cli  # noqa: F401
+    import nef_pipelines.transcoders.csv.importers._rdcs_cli  # noqa: F401
